@@ -46,6 +46,7 @@ struct _MetaMonitorInfo
   int number;
   MetaRectangle rect;
   gboolean is_primary;
+  gboolean in_fullscreen;
   XID output; /* The primary or first output for this crtc, None if no xrandr */
 };
 
@@ -117,6 +118,7 @@ struct _MetaScreen
   guint32 wm_cm_timestamp;
 
   guint work_area_later;
+  guint check_fullscreen_later;
 
   int rows_of_workspaces;
   int columns_of_workspaces;
@@ -201,6 +203,8 @@ void          meta_screen_get_natural_monitor_list (MetaScreen *screen,
 void          meta_screen_update_workspace_layout (MetaScreen             *screen);
 void          meta_screen_update_workspace_names  (MetaScreen             *screen);
 void          meta_screen_queue_workarea_recalc   (MetaScreen             *screen);
+void          meta_screen_queue_check_fullscreen  (MetaScreen             *screen);
+
 
 Window meta_create_offscreen_window (Display *xdisplay,
                                      Window   parent,
