@@ -32,6 +32,44 @@
 #include <gdesktop-enums.h>
 #include <gio/gio.h>
 
+/**
+ * MetaPreference:
+ * @META_PREF_MOUSE_BUTTON_MODS: mouse button modifiers
+ * @META_PREF_FOCUS_MODE: focus mode
+ * @META_PREF_FOCUS_NEW_WINDOWS: focus new windows
+ * @META_PREF_ATTACH_MODAL_DIALOGS: attach modal dialogs
+ * @META_PREF_RAISE_ON_CLICK: raise on click
+ * @META_PREF_ACTION_DOUBLE_CLICK_TITLEBAR: action double click titlebar
+ * @META_PREF_ACTION_MIDDLE_CLICK_TITLEBAR: action middle click titlebar
+ * @META_PREF_ACTION_RIGHT_CLICK_TITLEBAR: action right click titlebar
+ * @META_PREF_AUTO_RAISE: auto-raise
+ * @META_PREF_AUTO_RAISE_DELAY: auto-raise delay
+ * @META_PREF_FOCUS_CHANGE_ON_POINTER_REST: focus change on pointer rest
+ * @META_PREF_THEME: theme
+ * @META_PREF_TITLEBAR_FONT: title-bar font
+ * @META_PREF_NUM_WORKSPACES: number of workspaces
+ * @META_PREF_DYNAMIC_WORKSPACES: dynamic workspaces
+ * @META_PREF_APPLICATION_BASED: application-based
+ * @META_PREF_KEYBINDINGS: keybindings
+ * @META_PREF_DISABLE_WORKAROUNDS: disable workarounds
+ * @META_PREF_BUTTON_LAYOUT: button layout
+ * @META_PREF_WORKSPACE_NAMES: workspace names
+ * @META_PREF_VISUAL_BELL: visual bell
+ * @META_PREF_AUDIBLE_BELL: audible bell
+ * @META_PREF_VISUAL_BELL_TYPE: visual bell type
+ * @META_PREF_GNOME_ACCESSIBILITY: GNOME accessibility
+ * @META_PREF_GNOME_ANIMATIONS: GNOME animations
+ * @META_PREF_CURSOR_THEME: cursor theme
+ * @META_PREF_CURSOR_SIZE: cursor size
+ * @META_PREF_RESIZE_WITH_RIGHT_BUTTON: resize with right button
+ * @META_PREF_EDGE_TILING: edge tiling
+ * @META_PREF_FORCE_FULLSCREEN: force fullscreen
+ * @META_PREF_WORKSPACES_ONLY_ON_PRIMARY: workspaces only on primary
+ * @META_PREF_NO_TAB_POPUP: no tab popup
+ * @META_PREF_DRAGGABLE_BORDER_WIDTH: draggable border width
+ * @META_PREF_AUTO_MAXIMIZE: auto-maximize
+ */
+
 /* Keep in sync with GSettings schemas! */
 typedef enum
 {
@@ -138,6 +176,93 @@ int      meta_prefs_get_draggable_border_width (void);
 gboolean meta_prefs_get_ignore_request_hide_titlebar (void);
 void     meta_prefs_set_ignore_request_hide_titlebar (gboolean whether);
 
+/**
+ * MetaKeyBindingAction:
+ * @META_KEYBINDING_ACTION_NONE: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_1: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_2: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_3: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_4: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_5: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_6: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_7: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_8: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_9: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_10: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_11: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_12: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_LEFT: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_RIGHT: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_UP: FILLME 
+ * @META_KEYBINDING_ACTION_WORKSPACE_DOWN: FILLME 
+ * @META_KEYBINDING_ACTION_SWITCH_APPLICATIONS: FILLME 
+ * @META_KEYBINDING_ACTION_SWITCH_APPLICATIONS_BACKWARD: FILLME 
+ * @META_KEYBINDING_ACTION_SWITCH_GROUP: FILLME 
+ * @META_KEYBINDING_ACTION_SWITCH_GROUP_BACKWARD: FILLME 
+ * @META_KEYBINDING_ACTION_SWITCH_WINDOWS: FILLME 
+ * @META_KEYBINDING_ACTION_SWITCH_WINDOWS_BACKWARD: FILLME 
+ * @META_KEYBINDING_ACTION_SWITCH_PANELS: FILLME 
+ * @META_KEYBINDING_ACTION_SWITCH_PANELS_BACKWARD: FILLME 
+ * @META_KEYBINDING_ACTION_CYCLE_GROUP: FILLME 
+ * @META_KEYBINDING_ACTION_CYCLE_GROUP_BACKWARD: FILLME 
+ * @META_KEYBINDING_ACTION_CYCLE_WINDOWS: FILLME 
+ * @META_KEYBINDING_ACTION_CYCLE_WINDOWS_BACKWARD: FILLME 
+ * @META_KEYBINDING_ACTION_CYCLE_PANELS: FILLME 
+ * @META_KEYBINDING_ACTION_CYCLE_PANELS_BACKWARD: FILLME 
+ * @META_KEYBINDING_ACTION_TAB_POPUP_SELECT: FILLME 
+ * @META_KEYBINDING_ACTION_TAB_POPUP_CANCEL: FILLME 
+ * @META_KEYBINDING_ACTION_SHOW_DESKTOP: FILLME 
+ * @META_KEYBINDING_ACTION_PANEL_MAIN_MENU: FILLME 
+ * @META_KEYBINDING_ACTION_PANEL_RUN_DIALOG: FILLME 
+ * @META_KEYBINDING_ACTION_TOGGLE_RECORDING: FILLME 
+ * @META_KEYBINDING_ACTION_SET_SPEW_MARK: FILLME 
+ * @META_KEYBINDING_ACTION_ACTIVATE_WINDOW_MENU: FILLME 
+ * @META_KEYBINDING_ACTION_TOGGLE_FULLSCREEN: FILLME 
+ * @META_KEYBINDING_ACTION_TOGGLE_MAXIMIZED: FILLME 
+ * @META_KEYBINDING_ACTION_TOGGLE_TILED_LEFT: FILLME 
+ * @META_KEYBINDING_ACTION_TOGGLE_TILED_RIGHT: FILLME 
+ * @META_KEYBINDING_ACTION_TOGGLE_ABOVE: FILLME 
+ * @META_KEYBINDING_ACTION_MAXIMIZE: FILLME 
+ * @META_KEYBINDING_ACTION_UNMAXIMIZE: FILLME 
+ * @META_KEYBINDING_ACTION_TOGGLE_SHADED: FILLME 
+ * @META_KEYBINDING_ACTION_MINIMIZE: FILLME 
+ * @META_KEYBINDING_ACTION_CLOSE: FILLME 
+ * @META_KEYBINDING_ACTION_BEGIN_MOVE: FILLME 
+ * @META_KEYBINDING_ACTION_BEGIN_RESIZE: FILLME 
+ * @META_KEYBINDING_ACTION_TOGGLE_ON_ALL_WORKSPACES: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_1: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_2: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_3: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_4: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_5: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_6: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_7: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_8: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_9: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_10: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_11: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_12: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_LEFT: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_RIGHT: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_UP: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_DOWN: FILLME 
+ * @META_KEYBINDING_ACTION_RAISE_OR_LOWER: FILLME 
+ * @META_KEYBINDING_ACTION_RAISE: FILLME 
+ * @META_KEYBINDING_ACTION_LOWER: FILLME 
+ * @META_KEYBINDING_ACTION_MAXIMIZE_VERTICALLY: FILLME 
+ * @META_KEYBINDING_ACTION_MAXIMIZE_HORIZONTALLY: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_CORNER_NW: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_CORNER_NE: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_CORNER_SW: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_CORNER_SE: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_SIDE_N: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_SIDE_S: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_SIDE_E: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_SIDE_W: FILLME 
+ * @META_KEYBINDING_ACTION_MOVE_TO_CENTER: FILLME 
+ * @META_KEYBINDING_ACTION_OVERLAY_KEY: FILLME 
+ * @META_KEYBINDING_ACTION_LAST: FILLME 
+ */
 /* XXX FIXME This should be x-macroed, but isn't yet because it would be
  * difficult (or perhaps impossible) to add the suffixes using the current
  * system.  It needs some more thought, perhaps after the current system
@@ -232,6 +357,14 @@ typedef enum _MetaKeyBindingAction
   META_KEYBINDING_ACTION_LAST
 } MetaKeyBindingAction;
 
+/**
+ * MetaKeyBindingFlags:
+ * @META_KEY_BINDING_NONE: none
+ * @META_KEY_BINDING_PER_WINDOW: per-window
+ * @META_KEY_BINDING_BUILTIN: built-in
+ * @META_KEY_BINDING_REVERSES: reverses
+ * @META_KEY_BINDING_IS_REVERSED: is reversed
+ */
 typedef enum
 {
   META_KEY_BINDING_NONE,
@@ -241,16 +374,28 @@ typedef enum
   META_KEY_BINDING_IS_REVERSED = 1 << 3
 } MetaKeyBindingFlags;
 
-typedef struct
+/**
+ * MetaKeyCombo:
+ * @keysym: keysym
+ * @keycode: keycode
+ * @modifiers: modifiers
+ */
+typedef struct _MetaKeyCombo MetaKeyCombo;
+struct _MetaKeyCombo
 {
   unsigned int keysym;
   unsigned int keycode;
   MetaVirtualModifier modifiers;
-} MetaKeyCombo;
+};
 
 /**
  * MetaKeyHandlerFunc:
- * @event: (type gpointer):
+ * @display: a #MetaDisplay
+ * @screen: a #MetaScreen
+ * @window: a #MetaWindow
+ * @event: (type gpointer): a #XIDeviceEvent
+ * @binding: a #MetaKeyBinding
+ * @user_data: data passed to the function
  *
  */
 typedef void (* MetaKeyHandlerFunc) (MetaDisplay    *display,
